from marshmallow import ValidationError
from marshmallow import validate as ma_validate
import marshmallow as ma
from marshmallow_utils import fields as mu_fields
from marshmallow_utils import schemas as mu_schemas

{{ imports|generate_import }}

{% for cls in generated_classes %}
class {{cls.class_name|base_name}}({% for bc in cls.base_classes %}{% if not loop.first%}, {% endif %}{{bc}}{% endfor %}):
    class Meta:
        unknown = ma.{{ cls.unknown }}
{% for fld in cls.fields %}
    {{fld.key}} = {{fld.full_definition}}
{% endfor %}
{% endfor %}
