"""Facet definitions."""

from invenio_search.engine import dsl
from flask_babelex import lazy_gettext as _

{{ imports|generate_import }}


{% for facet in facets %}
{{ facet.path }} = {{ facet.field }}
{% endfor %}

{#{{ vars.invenio_record_facets_extra_code }}#}